/**
 * @file IRremote2.hpp
*/

#ifndef IRremote2_hpp
#define IRremote2_hpp

#define VERSION_IRREMOTE "3.5.0"
#define VERSION_IRREMOTE_MAJOR 3
#define VERSION_IRREMOTE_MINOR 5

/****************************************************
 *                     PROTOCOLS
 ****************************************************/
/*
 * Supported IR protocols
 * Copy the lines with the protocols you need in your program before the #include <IRremote2.hpp> line
 */

#define DECODE_NEC          // Includes Apple and Onkyo

//#define DEBUG // Activate this for lots of lovely debug output from the IRremote core.

/****************************************************
 *                    RECEIVING
 * The length of the buffer where the IR timing data is stored before decoding
 * 100 is sufficient for most standard protocols
 */

#define RAW_BUFFER_LENGTH  100  // Length of raw duration buffer. Must be even. 100 supports up to 48 bit codings inclusive 1 start and 1 stop bit.

/**
 * MARK_EXCESS_MICROS is subtracted from all marks and added to all spaces before decoding,
 * to compensate for the signal forming of different IR receiver modules
 * For Vishay TSOP*, marks tend to be too long and spaces tend to be too short.
 * If you set MARK_EXCESS_MICROS to approx. 50us then the TSOP4838 works best.
 * At 100us it also worked, but not as well.
 * Set MARK_EXCESS to 100us and the VS1838 doesn't work at all.
 *
 * The right value is critical for IR codes using short pulses like Denon / Sharp / Lego
 *
 *  Observed values:
 *  Delta of each signal type is around 50 up to 100 and at low signals up to 200. TSOP is better, especially at low IR signal level.
 *  VS1838      Mark Excess -50 to +50 us
 *  TSOP31238   Mark Excess 0 to +50
 */
#define MARK_EXCESS_MICROS    20

/**
 * Minimum gap between IR transmissions, to detect the end of a protocol.
 * Must be greater than any space of a protocol e.g. the NEC header space of 4500 us.
 * Must be smaller than any gap between a command and a repeat; e.g. the retransmission gap for Sony is around 24 ms.
 * Keep in mind, that this is the delay between the end of the received command and the start of decoding.
 */
#define RECORD_GAP_MICROS   5000 // FREDRICH28AC / LG2 header space is 9700, NEC header space is 4500

/**
 * Threshold for warnings at printIRResult*() to report about changing the RECORD_GAP_MICROS value to a higher value.
 */
#define RECORD_GAP_MICROS_WARNING_THRESHOLD   20000

/** Minimum gap between IR transmissions, in MICROS_PER_TICK */
#define RECORD_GAP_TICKS    (RECORD_GAP_MICROS / MICROS_PER_TICK) // 221 for 1100

#define INPUT_MARK   0 ///< Sensor output for a mark ("flash")

#include "private/IRTimer2.hpp"  //needed for receive

#include "IRFeedbackLED2.hpp"  //needed for receiver to compile

/*
 * Include the sources here to enable compilation with macro values set by user program.
 */
#include "IRReceive2.hpp"

// Includes Apple and Onkyo
#include "ir_NEC2.hpp"

#endif // IRremote_hpp2

#pragma once

